<?php
require_once 'lib/vendor/autoload.php';

use \Vtbpay\Classes\Api\VtbApi,
    \Vtbpay\Classes\Common\VtbPayLogger,
    \Vtbpay\Classes\Exception\VtbPayException;

class Pactioner extends Actioner
{
    const PLUGIN_VERSION = '1.2.4';

    /**
     * The name of the plugin.
     * @var string $pluginName
     */
    private static $pluginName = 'vtbpay-payment';

    /**
     * Logger object.
     * @var VtbPayLogger
     */
    private $logger;

    /**
     * Payment parameters
     *
     * @var array
     */
    private $paymentParams;


    /**
     * getVtbApi
     *
     * Initialize and return an instance of the VtbApi with provided payment parameters.
     *
     * @return VtbApi Instance of VtbApi with provided payment parameters.
     */
    private function getVtbApi()
    {
        $paymentParams = $this->paymentParams;
        return new VtbApi(
            $paymentParams['client_id'],
            $paymentParams['client_secret'],
            filter_var($paymentParams['test_mode'], FILTER_VALIDATE_BOOLEAN),
            $paymentParams['merchant_authorization'] ?: ''
        );
    }


    /**
     * getPayLink
     *
     * Generate a payment link for a given order. Handles exceptions and logs payment events.
     *
     * @return string|bool The generated payment link or false if an error occurred.
     */
    public function getPayLink()
    {
        $paymentId = $_POST['paymentId'];
        $mgBaseDir = $_POST['mgBaseDir'];
        $orderNumber = str_replace('№ ', '', $_POST['orderNumber']);

        $this->setVtbpayLogger();
        $this->logger->setOption('additionalCommonText', 'payment-' . rand(1111, 9999));

        try {
            if (empty($orderNumber)) {
                throw new \Exception($this->lang['VTBPAY_ORDER_NUMBER_IS_EMPTY']);
            }

            $order = $this->getOrderByNumber($orderNumber);
            if (!$order) {
                throw new \Exception($this->lang['VTBPAY_ORDER_NOT_FOUND']);
            }

            $paymentId = $paymentId ?: $order['payment_id'];

            $total = (float) $this->getOrderTotal($order);

            $returnUrl = $mgBaseDir . '/ajaxrequest?mguniqueurl=action/notification&pluginHandler=' .
                         self::$pluginName . '&payment=' .
                         $paymentId;

            $this->setPaymentParams($paymentId);

            $result = $this->getPayUrl(
                $order,
                $total,
                $returnUrl
            );

            return $this->data['result'] = $result;

        } catch (Exception | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            // Log the caught exception for debugging purposes.
            $this->logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order id: %s;',
                $e->getMessage(),
                $orderNumber
            ), $context);

            return false;
        }
    }


    /**
     * Получает URL-адрес платежа для перенаправления.
     *
     * @return string URL-адрес платежа.
     */

    private function getPayUrl($order, $total, $returnUrl)
    {
        $paymentParams = $this->paymentParams;

        $enableFiscal = (bool) ($paymentParams['enable_fiscal'] ?? false);
        $items = [];
        if ($enableFiscal) $items = $this->getItems($order);

        $email = $order['user_email'] ?: $paymentParams['email_fiscal'] ?? '';

        // Отправляем название cms и версию плагина
        $appendedOrderName = 'CMS: Moguta. ' . $this->lang['VTBPAY_PLUGIN_VERSION'] . ': ' . self::PLUGIN_VERSION;

        // Logging input
        $this->logger->debug(
            basename(__FILE__) . ' > ' . __FUNCTION__ . ' > getOrderLink - INPUT: ', [
            'arguments' => [
                'id' => $order['id'],
                'orderNumber' => $order['number'],
                'email' => $email,
                'time' => time(),
                'total' => $total,
                'returnUrl' => $returnUrl,
                'twoStage' => false,
                'items' => $items,
                'appendedOrderName' => $appendedOrderName
            ]
        ]);

        $response = $this->getVtbApi()->getOrderLink(
            $order['number'],
            $email,
            time(),
            $total,
            $returnUrl,
            false,
            $items,
            $appendedOrderName
        );

        $orderCode = $response['object']['orderCode'] ?? '';
        if (!empty($orderCode)) $this->addTransaction($order['id'], $orderCode);

        // Logging output
        $this->logger->debug(
            basename(__FILE__) . ' > ' . __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
            'response' => $response
        ]);

        return $response['object']['payUrl'] ?? '';
    }


    /**
     * notification
     *
     * Receives a notification of a payment event, validates it and processes accordingly.
     * Redirects to success or failure URLs based on the payment status.
     *
     * @return void
     */
    public function notification()
    {
        $paymentId = $_GET['payment'];
        $orderNumber = $_GET['orderId'];
        $this->data['result'] = $orderNumber;

        $this->setVtbpayLogger();
        $this->logger->setOption('additionalCommonText', 'return-' . rand(1111, 9999));

        try {
            if (empty($orderNumber)) {
                throw new \Exception($this->lang['VTBPAY_ORDER_NUMBER_IS_EMPTY']);
            }

            $order = $this->getOrderByNumber($orderNumber);
            if (!$order) {
                throw new \Exception($this->lang['VTBPAY_ORDER_NOT_FOUND']);
            }

            $operationId = $this->getOperationId($order['id']);
            if (!$operationId) {
                throw new \Exception($this->lang['VTBPAY_OPERATION_NOT_FOUND']);
            }

            $this->setPaymentParams($paymentId);

            $paymentStatusData = $this->getPaymentStatusData($orderNumber);

            if ($paymentStatusData['object']['status']['value'] !== 'PAID') {
                throw new \Exception($this->lang['VTBPAY_PAYMENT_NOT_FOUND']);
            }

            $payment = new Controllers_Payment();
            $payment->actionWhenPayment([
                'paymentOrderId' => $order['id'],
                'paymentAmount' => $paymentStatusData['object']['amount']['value'],
                'paymentID' => $paymentId
            ]);

            $redirectUrlGetParams = '&payStatus=success&id=' . $paymentId .
                '&orderId=' . $orderNumber;

        } catch (Exception | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            // Log the caught exception for debugging purposes.
            $this->logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order id: %s;',
                $e->getMessage(),
                $orderNumber ?: ''
            ), $context);

            $redirectUrlGetParams = '&payStatus=fail';
            if (
                !empty($paymentId) &&
                !empty($orderNumber)
            ) {
                $redirectUrlGetParams = '&payStatus=fail&id=' . $paymentId .
                    '&orderId=' . $orderNumber;
            }
        }

        MG::redirect('/payment?plugin_name=' . self::$pluginName . $redirectUrlGetParams);
    }


    /**
     * getPaymentStatus() method requests the payment status from the VtbApi
     * for a specific order and returns it, throwing an exception if no status is received.
     *
     * @param string $orderNumber Order number
     * @return array Returns payment status data
     */
    public function getPaymentStatusData($orderNumber)
    {
        // Logging input
        $this->logger->debug(
            basename(__FILE__) . ' > ' . __FUNCTION__ . ' > getOrderInfo - INPUT: ', [
            'arguments' => [
                'orderNumber' => $orderNumber
            ]
        ]);

        $response = $this->getVtbApi()->getOrderInfo($orderNumber);

        // Logging output
        $this->logger->debug(
            basename(__FILE__) . ' > ' . __FUNCTION__ . ' > getOrderInfo - OUTPUT: ', [
            'response' => $response
        ]);

        return $response ?: [];
    }


    /**
     * Получает массив товаров заказа.
     *
     * @return array Массив товаров.
     */

    private function getItems($order): array
    {
        $getAllItems = unserialize(stripslashes($order['order_content']));
        $paymentParams = $this->paymentParams;

        $items = [];
        foreach ($getAllItems as $key => $item) {
            $items[] = [
                'positionId' => ($key + 1),
                'name' => $item['name'],
                'code' => $item['code'] ?? (string) $item['id'],
                'price' => floatval($item['fulPrice']),
                'measure' => (int) $paymentParams['measure_fiscal'] ?? 0,
                'quantity' => (int) $item['count'],
                'taxParams' => [
                    'taxType' => $paymentParams['tax_type_fiscal'] ?? 'none'
                ],
                'paymentType' => $paymentParams['payment_type_fiscal'] ?? 'full_prepayment',
                'paymentSubject' => (int) $paymentParams['payment_subject_fiscal'] ?? 1,
                'amount' => floatval($item['fulPrice'] * $item['count'])
            ];
        }

        $deliveryCost = floatval($order['delivery_cost'] ?? 0);
        // Стоимость доставки
        if ($deliveryCost > 0) {
            $items[] = [
                'positionId' => ($key + 2),
                'name' => 'Доставка',
                'price' => $deliveryCost,
                'measure' => 0,
                'quantity' => 1,
                'taxParams' => [
                    'taxType' => $paymentParams['tax_type_fiscal'] ?? 'none'
                ],
                'paymentType' => $paymentParams['payment_type_delivery_fiscal'] ?? 'full_prepayment',
                'paymentSubject' => 4,
                'amount' => $deliveryCost
            ];
        }

        return $items;
    }


    /**
     * getPaymentData
     *
     * Fetches the payment information from the database using the provided payment ID.
     *
     * @param int $paymentId The ID of the payment to fetch.
     * @return array An associative array of the payment data.
     */
    private function getPaymentData($paymentId)
    {
        $dbRes = DB::query("SELECT *
									          FROM `{$this->prefix}payment`
									          WHERE `id` = '{$paymentId}'");
        return DB::fetchArray($dbRes);
    }


    /**
     * getOrder
     *
     * Fetches an order from the database using the provided order ID.
     *
     * @param int $orderId The ID of the order to fetch.
     * @return array An associative array of the order data.
     */
    private function getOrder($orderId)
    {
        $dbRes = DB::query("SELECT *
									          FROM `{$this->prefix}order`
									          WHERE `id` = '{$orderId}'");
        return DB::fetchAssoc($dbRes);
    }


    /**
     * getOrderByNumber
     *
     * Fetches an order from the database using the provided order number.
     *
     * @param int $orderNumber The number of the order to fetch.
     * @return array An associative array of the order data.
     */
    private function getOrderByNumber($orderNumber)
    {
        $dbRes = DB::query("SELECT *
									          FROM `{$this->prefix}order`
									          WHERE `number` = '{$orderNumber}'");
        return DB::fetchAssoc($dbRes);
    }


    /**
     * setPaymentParams
     *
     * Sets the class-level payment parameters variable using the provided payment ID.
     *
     * @param int $paymentId The ID of the payment.
     * @return void
     */
    private function setPaymentParams($paymentId)
    {
        $this->paymentParams = Models_Payment::getPaymentParams(self::$pluginName, true);
    }


    /**
     * getOrderTotal
     *
     * Calculates and returns the total cost of an order, including delivery cost if available.
     *
     * @param array $order The order to calculate the total for.
     * @return float The total cost of the order.
     */
    private function getOrderTotal($order)
    {
        if (
            isset($order['delivery_cost']) &&
            $order['delivery_cost'] > 0
        ) {
            $summ = $order['summ'] + $order['delivery_cost'];
        }
        else {
            $summ = $order['summ'];
        }

        return $summ;
    }


    /**
     * addTransaction
     *
     * Adds a new transaction to the database with the provided order ID and operation ID.
     *
     * @param int $orderNumber The number of the order associated with the transaction.
     * @param int $operationId The ID of the operation associated with the transaction.
     * @return void
     */
    private function addTransaction($id, $operationId)
    {
        $sql = "INSERT INTO `{$this->prefix}vtbpay_transactions`
								VALUES ('{$id}', '{$operationId}', CURRENT_TIMESTAMP);";
        DB::query($sql);
    }


    /**
     * getOperationId
     *
     * Fetches the operation ID associated with the provided order ID from the database.
     *
     * @param int $orderNumber The number of the order to fetch the operation for.
     * @return string The operation ID associated with the order.
     */
    private function getOperationId($id)
    {
        $dbRes = DB::query("SELECT *
			          FROM `{$this->prefix}vtbpay_transactions`
			          WHERE `id` = '{$id}'
								ORDER BY `date` DESC");
        $result = DB::fetchAssoc($dbRes);

        return $result['transaction_id'];
    }


    /**
     * Устанавливает объект логирования для модуля.
     */
    private function setVtbpayLogger()
    {
        $logger = VtbPayLogger::getInstance();

        $this->logger = $logger
            ->setCustomRecording(function($message) use ($wc_log){
                MG::loger($message,'append','vtbpay');
            }, VtbPayLogger::LOG_LEVEL_ERROR)
            ->setCustomRecording(function($message) use ($logger) {

                $logging = (bool) ($this->paymentParams['logging'] ?? false);
                if ($logging) MG::loger($message,'append','vtbpay');
            }, VtbPayLogger::LOG_LEVEL_DEBUG);
    }
}
